/*
 * Copyright 2016 - 2024 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#pragma once

#include <ox/fs/fs.hpp>
#include <ox/model/desctypes.hpp>
#include <ox/std/buffer.hpp>
#include <ox/std/size.hpp>

#include <keel/context.hpp>

#include "input.hpp"

namespace turbine {

class Context;

struct ContextDeleter {
	void operator()(Context *p) noexcept;
};

using ContextUPtr = ox::UPtr<Context, ContextDeleter>;

void shutdown(Context &ctx) noexcept;

keel::Context const&keelCtx(Context const&ctx) noexcept;

keel::Context &keelCtx(Context &ctx) noexcept;

inline ox::FileSystem const*rom(Context const&ctx) noexcept {
	return keelCtx(ctx).rom.get();
}

inline ox::FileSystem *rom(Context &ctx) noexcept {
	return keelCtx(ctx).rom.get();
}

struct WrapBase {
	virtual ~WrapBase() = default;
};

template<typename T>
struct Wrap: public WrapBase {
	T *data{};
};

void setApplicationDataRaw(Context &ctx, void *applicationData) noexcept;

template<typename T>
void setApplicationData(Context &ctx, T *applicationData) noexcept {
	Wrap<T> w;
	w.data = applicationData;
	setApplicationDataRaw(ctx, &w);
}

[[nodiscard]]
WrapBase *applicationDataRaw(Context &ctx) noexcept;

template<typename T>
[[nodiscard]]
T *applicationData(Context &ctx) noexcept {
#ifdef OX_BARE_METAL
	return static_cast<Wrap<T>*>(applicationDataRaw(ctx))->data;
#else
	auto const out = dynamic_cast<Wrap<T>*>(applicationDataRaw(ctx));
	oxAssert(out, "Cast failed - wrong type");
	return out->data;
#endif
}

void setKeyEventHandler(Context &ctx, KeyEventHandler h) noexcept;

KeyEventHandler keyEventHandler(Context &ctx) noexcept;

}

